﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;
using ReportWriterSample;
using ReportWriterSample.UWP.Print;
using System.IO;
using Windows.Storage;
using Windows.UI.Xaml.Printing;
using Windows.Graphics.Printing;
using Windows.UI.Xaml;
using Windows.Data.Pdf;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Media;
using Windows.Storage.Streams;

[assembly: Dependency(typeof(DependencyServiceUWP))]
namespace ReportWriterSample.UWP.Print
{
    public class DependencyServiceUWP : IDependencyServiceInterface
    {
        private bool printResult = false;

        private PrintDocument printDocument = null;
        private IPrintDocumentSource printDocumentSource = null;
        private List<UIElement> printPreviewPages = new List<UIElement>();

        Windows.UI.Xaml.Controls.Page printPage;
        Windows.UI.Xaml.Controls.Frame frame;

        private void Register()
        {
            // Create the PrintDocument.
            printDocument = new PrintDocument();

            // Save the DocumentSource.
            printDocumentSource = printDocument.DocumentSource;

            // Add an event handler which creates preview pages.
            printDocument.Paginate += PrintDocument_Paginate;

            // Add an event handler which provides a specified preview page.
            printDocument.GetPreviewPage += PrintDocument_GetPrintPreviewPage;

            // Add an event handler which provides all final print pages.
            printDocument.AddPages += PrintDocument_AddPages;

            // Create a PrintManager and add a handler for printing initialization.
            PrintManager printMan = PrintManager.GetForCurrentView();
            printMan.PrintTaskRequested += PrintManager_PrintTaskRequested;
        }

        private void UnRegister()
        {
            printDocument.Paginate -= PrintDocument_Paginate;
            printDocument.GetPreviewPage -= PrintDocument_GetPrintPreviewPage;
            printDocument.AddPages -= PrintDocument_AddPages;
            PrintManager printMan = PrintManager.GetForCurrentView();
            printMan.PrintTaskRequested -= PrintManager_PrintTaskRequested;
        }

        private async void PreparePrintContent(MemoryStream stream)
        {
            frame = Window.Current.Content as Windows.UI.Xaml.Controls.Frame;
            StorageFile outFile = await ApplicationData.Current.TemporaryFolder.CreateFileAsync("exportReport.pdf", CreationCollisionOption.GenerateUniqueName);

            using (Stream outStream = await outFile.OpenStreamForWriteAsync())
            {
                using (var memoryStream = new MemoryStream())
                {
                    stream.CopyTo(memoryStream);
                    outStream.Write(memoryStream.ToArray(), 0, (int)stream.Length);
                }
            }

            if (outFile != null)
            {
                IRandomAccessStream randomStream = await outFile.OpenAsync(FileAccessMode.ReadWrite);
                await outFile.DeleteAsync();

                randomStream.Seek(0);

                PdfDocument pdfDoc = await PdfDocument.LoadFromStreamAsync(randomStream);
                printPreviewPages.Clear();

                for (uint i = 0; i < pdfDoc.PageCount; i++)
                {
                    var bitmap = new BitmapImage();
                    MemoryStream mStream = new MemoryStream();
                    var outStream = mStream.AsRandomAccessStream();
                    await pdfDoc.GetPage(i).RenderToStreamAsync(outStream, new PdfPageRenderOptions());
                    bitmap.SetSource(outStream);
                    Windows.UI.Xaml.Controls.Image image = new Windows.UI.Xaml.Controls.Image();
                    image.Stretch = Stretch.Fill;
                    image.Source = bitmap;
                    printPage = new Windows.UI.Xaml.Controls.Page();
                    printPage.Content = image;
                    printPreviewPages.Add(printPage);
                }
            }
        }
        private void PrintDocument_Paginate(object sender, PaginateEventArgs e)
        {
            PrintDocument printDoc = (PrintDocument)sender;
            // Report the number of preview pages created
            printDoc.SetPreviewPageCount(printPreviewPages.Count, PreviewPageCountType.Intermediate);
        }

        private void PrintDocument_GetPrintPreviewPage(object sender, GetPreviewPageEventArgs e)
        {
            PrintDocument printDoc = (PrintDocument)sender;

            printDoc.SetPreviewPage(e.PageNumber, printPreviewPages[e.PageNumber - 1]);
        }

        private void PrintDocument_AddPages(object sender, AddPagesEventArgs e)
        {
            // Loop over all of the preview pages and add each one to  add each page to be printied
            for (int i = 0; i < printPreviewPages.Count; i++)
            {
                // We should have all pages ready at this point...
                printDocument.AddPage(printPreviewPages[i]);
            }

            PrintDocument printDoc = (PrintDocument)sender;

            // Indicate that all of the print pages have been provided
            printDoc.AddPagesComplete();
        }

        private void PrintManager_PrintTaskRequested(PrintManager sender, PrintTaskRequestedEventArgs e)
        {
            PrintTask printTask = null;
            printTask = e.Request.CreatePrintTask("Printed output.", sourceRequested =>
            {
                // Print Task event handler is invoked when the print job is completed.
                printTask.Completed += async (s, args) =>
                {
                    await frame.Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
                    {
                        UnRegister();
                    });

                    //Notify the user when the print operation fails.
                    if (args.Completion == PrintTaskCompletion.Failed)
                    {
                        printResult = false;
                    }
                };

                sourceRequested.SetSource(printDocumentSource);
            });
        }
     

        public async Task<bool> PrintPdfAsync(MemoryStream stream)
        {
            Register();
            this.PreparePrintContent(stream);
            await Windows.Graphics.Printing.PrintManager.ShowPrintUIAsync();

            return this.printResult;
        }

        public async Task<bool> SavePdfAsync(MemoryStream stream)
        {
            try
            {
                StorageFolder local = Windows.Storage.ApplicationData.Current.LocalFolder;

                StorageFile outFile = await local.CreateFileAsync("exportReport.pdf", CreationCollisionOption.GenerateUniqueName);

                using (Stream outStream = await outFile.OpenStreamForWriteAsync())
                {
                    using (var memoryStream = new MemoryStream())
                    {
                        stream.Position = 0;
                        stream.CopyTo(memoryStream);
                        outStream.Write(memoryStream.ToArray(), 0, (int)memoryStream.Length);
                    }
                }

                await Windows.System.Launcher.LaunchFileAsync(outFile);

                return true;
            }
            catch (Exception e)
            {
            }

            return false;
        }
    }
}

